@@ -19,18 +19,26 @@
 
 #pragma once
 
+#include "client/keeper-errors.h"
 #include "helper/metadata.h"
 
+#include <QObject>
 #include <QVector>
 
-class MetadataProvider
+class MetadataProvider : public QObject
 {
+    Q_OBJECT
 public:
     virtual ~MetadataProvider() =0;
     virtual QVector<Metadata> get_backups() const =0;
+    virtual void get_backups_async(QString const & storage) =0;
+
+Q_SIGNALS:
+    void finished(keeper::Error error);
 
 protected:
-    MetadataProvider() =default;
+    explicit MetadataProvider(QObject *parent = nullptr) : QObject(parent){};
+    QVector<Metadata> backups_;
 };
 
 inline MetadataProvider::~MetadataProvider() =default;
